@@ -1,6 +1,6 @@
 fuzzy.ts1 <-
 function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh", 
-    "Heuristic", "Chen-Hsu"), bin = NULL, trace = FALSE, plot = FALSE) 
+    "Heuristic", "Chen-Hsu"), bin = NULL, trace = FALSE, plot = FALSE,grid=FALSE) 
 {
     is.wholenumber <- function(x, tol = .Machine$double.eps^0.5) abs(x - 
         round(x)) < tol
@@ -158,11 +158,11 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
         kt = kt + 1
     if (kt > 0) 
         stop("Trong chuoi co gia tri NA!")
-    if (is.na(D1) || !is.numeric(D1)) 
+    if (is.na(D1) || !is.numeric(D1) || length(D1)>1) 
         stop("Error in 'D1'!")
-    if (is.na(D2) || !is.numeric(D2)) 
+    if (is.na(D2) || !is.numeric(D2) || length(D2)>1) 
         stop("Error in 'D2'!")
-    if (is.na(n) || !is.numeric(n) || n < 1 || !is.wholenumber(n)) 
+    if (is.na(n) || !is.numeric(n) || n < 1 || !is.wholenumber(n) ||length(n)>1) 
         stop("Error in 'n'!")
     type <- match.arg(type)
     if (type != "Chen" & type != "Singh" & type != "Heuristic" & 
@@ -173,14 +173,20 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
             stop("'bin' just for Chen-Hsu model!")
     if (plot != 0 & plot != 1) 
         stop("Error in 'plot'!")
+    if (grid != 0 & grid != 1) 
+        stop("Error in 'grid'!")
     if (trace != 0 & trace != 1) 
-        stop("Error in 'plot'!")
+        stop("Error in 'trace'!")
+#---------------------------------------------
     d1 = D1
     d2 = D2
     thoidiem <- namthang(ts)
     quanhemo <- quanhe(ts, type)
     min.x = min(ts) - d1
     max.x = max(ts) + d2
+
+#---------------------------------------------
+if(type=="Chen" | type=="Singh" | type == "Heuristic" | (type=="Chen-Hsu" & is.null(bin))){
     h = (max.x - min.x)/n
     k <- 1:(n + 1)
     U <- 1:n
@@ -244,6 +250,29 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
         for (j in 1:n) if (a[j, cot] != paste("A", j, sep = "")) 
             a[j, cot] <- NA
     }
+
+#-------------------
+#Quan he mo
+qh.mo<-0
+for(qh in 1:n){
+a.qh<-as.character(na.omit(a[,qh]))
+
+a.qh1<-paste("A",qh,"->",sep="")
+for(noi in 1:length(a.qh)){
+if(noi==1) a.qh1<-paste(a.qh1,a.qh[noi],sep="")
+if(noi!=1) a.qh1<-paste(a.qh1,a.qh[noi],sep=",")
+}
+qh.mo<-rbind(qh.mo,a.qh1)
+}
+rownames(qh.mo)<-NULL
+qh.mo<-qh.mo[-1,]
+#-------------------
+}
+
+
+
+
+
     if (type == "Chen") {
         mo <- 1:n
         for (cot in 1:n) {
@@ -439,6 +468,8 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
                   l = i
                   break
                 }
+
+
                 if (l != 0) {
                   t = 0
                   for (i in l:length(locate)) t = t + D$Bw[locate[i]]
@@ -449,10 +480,13 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
                 }
             }
         }
+
+
         tap <- factor(tap[2:length(tap)])
         t.thai <- t.thai[2:length(t.thai)]
         Bw.h <- Bw.h[2:length(Bw.h)]
         D.h <- data.frame(tap, t.thai, Bw.h)
+
         Heuristic <- 1:length(ts)
         Heuristic[1] <- NA
         for (j in 2:length(ts)) {
@@ -528,7 +562,46 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
                 tt = tt + 1
                 v[tt] = t0
             }
-            if (tt == 0) 
+
+
+
+#-------------------
+ a <- 1:(n2 * n2)
+    a <- matrix(a, nrow = n2)
+    for (cot in 1:n2) {
+        for (i in 2:length(D1.ch$loai2.old)) {
+            for (j in 1:n2) {
+                if (D1.ch$loai2.old[i] == paste("A", cot, sep = "") & 
+                  a[j, cot] != paste("A", j, sep = "") & D1.ch$loai2[i] == 
+                  paste("A", j, sep = "")) {
+                  a[j, cot] = paste("A", j, sep = "")
+                  break
+                }
+            }
+        }
+    }
+    for (cot in 1:n2) {
+        for (j in 1:n2) if (a[j, cot] != paste("A", j, sep = "")) 
+            a[j, cot] <- NA
+    }
+
+#----------
+#Quan he mo
+qh.mo<-0
+for(qh in 1:n2){
+a.qh<-as.character(na.omit(a[,qh]))
+
+a.qh1<-paste("A",qh,"->",sep="")
+for(noi in 1:length(a.qh)){
+if(noi==1) a.qh1<-paste(a.qh1,a.qh[noi],sep="")
+if(noi!=1) a.qh1<-paste(a.qh1,a.qh[noi],sep=",")
+}
+qh.mo<-rbind(qh.mo,a.qh1)
+}
+rownames(qh.mo)<-NULL
+qh.mo<-qh.mo[-1,]
+#-------------------
+        if (tt == 0) 
                 (v = NULL)
             else v = v[1:tt]
             if (!is.null(v)) {
@@ -547,6 +620,7 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
                 }
             }
             else Dt <- D.ch
+
             X = ts
             A <- as.character(D1.ch$loai2)
             P <- 1:length(X)
@@ -608,14 +682,19 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
     if (type == "Chen-Hsu" & !is.null(bin)) 
         accuracy <- av.res(Y = data.frame(DB4[, 2]), F = data.frame(`Chen-Hsu` = DB4[, 
             4]))
+
     if (trace == TRUE) {
         namescot1 <- "set"
         namescot2 <- "dow"
         namescot3 <- "up"
         namescot4 <- "mid"
         namescot5 <- "num"
-        colnames(D) <- c(namescot1, namescot2, namescot3, namescot4, 
-            namescot5)
+
+if(type=="Chen" | type == "Singh" | type=="Heuristic" | (type=="Chen-Hsu" & is.null(bin)))
+colnames(D) <- c(namescot1, namescot2, namescot3, namescot4, namescot5)
+if(type=="Chen-Hsu" & !is.null(bin))
+colnames(D.ch) <- c(namescot1, namescot2, namescot3, namescot4, namescot5)
+
         namescot1 <- "point"
         namescot2 <- "ts"
         namescot3 <- "relative"
@@ -632,31 +711,29 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
         if (type == "Chen-Hsu" & !is.null(bin)) 
             colnames(DB4) <- c(namescot1, namescot2, namescot3, 
                 namescot4)
+
         if (type == "Chen") 
             MO <- list(type = "Chen", table1 = D, table2 = DB1, 
-                accuracy = accuracy)
+                relative.groups= qh.mo,accuracy = accuracy)
         if (type == "Singh") 
             MO <- list(type = "Singh", table1 = D, table2 = DB2, 
-                accuracy = accuracy)
+                relative.groups= qh.mo,accuracy = accuracy)
         if (type == "Heuristic") 
             MO <- list(type = "Heuristic", table1 = D, table2 = DB3, 
-                accuracy = accuracy)
+                relative.groups= qh.mo,accuracy = accuracy)
         if (type == "Chen-Hsu") {
             if (is.null(bin)) 
                 MO <- list(type = "Chen-Hsu", table1 = D)
             else if (!is.null(bin)) 
                 MO <- list(type = "Chen-Hsu", table1 = D.ch, 
-                  table2 = DB4, accuracy = accuracy)
+                  table2 = DB4, relative.groups= qh.mo,accuracy = accuracy)
         }
+
+if(type=="Chen" | type=="Singh" | type == "Heuristic" | (type=="Chen-Hsu" & !is.null(bin))){
+names(MO$table2)<-c("point","actual","relative","forecasted")}
+
     }
     else if (trace == FALSE) {
-        namescot1 <- "set"
-        namescot2 <- "dow"
-        namescot3 <- "up"
-        namescot4 <- "mid"
-        namescot5 <- "num"
-        colnames(D) <- c(namescot1, namescot2, namescot3, namescot4, 
-            namescot5)
         if (type == "Chen") 
             MO <- DB1[, 4]
         if (type == "Singh") 
@@ -665,138 +742,98 @@ function (ts, n = 5, D1 = 0, D2 = 0, type = c("Chen", "Singh",
             MO <- DB3[, 4]
         if (type == "Chen-Hsu" & !is.null(bin)) 
             MO <- DB4[, 4]
-        if (type == "Chen-Hsu" & is.null(bin)) 
+        if (type == "Chen-Hsu" & is.null(bin)){ 
+        namescot1 <- "set"
+        namescot2 <- "dow"
+        namescot3 <- "up"
+        namescot4 <- "mid"
+        namescot5 <- "num"
+        colnames(D) <- c(namescot1, namescot2, namescot3, namescot4, 
+            namescot5)
             MO <- list(type = "Chen-Hsu", table1 = D)
+}
     }
     else MO <- c("trace must be 'TRUE' or 'FALSE'")
-    if (plot == TRUE) {
-        if (type == "Chen") {
-            goc <- DB1[, 2]
-            dubao <- DB1[, 4]
-            n.dothi <- prod(par()$mfrow)
-            n.dothi
-            if (length(goc) < 50) {
-                plot(goc, col = "blue", main = paste("Chen", 
-                  n, "fuzzy set"), type = "o", pch = 16, ylim = c(min(c(goc, 
-                  dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                  xlab = "index", ylab = "data")
-                lines(dubao, col = "red", type = "o", pch = 18)
-                legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                  col = c("blue", "red"), lty = c(1, 1), pch = c(16, 
-                    18), cex = 1/n.dothi)
-            }
-            if (length(goc) > 49) {
-                plot(goc, col = "blue", main = paste("Chen", 
-                  n, "fuzzy set"), type = "l", ylim = c(min(c(goc, 
-                  dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                  xlab = "index", ylab = "data")
-                lines(dubao, col = "red", type = "l")
-                legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                  col = c("blue", "red"), lty = c(1, 1), cex = 1/n.dothi)
-            }
-            if (n.dothi == 1) {
-                k.ve <- par()$yaxp
-                h = (k.ve[2] - k.ve[1])/k.ve[3]
-                for (i in 0:k.ve[3]) abline(h = k.ve[1] + h * 
-                  i, lty = 3, col = "gray")
-            }
-        }
-        if (type == "Singh") {
-            goc <- DB2[, 2]
-            dubao <- DB2[, 4]
-            n.dothi <- prod(par()$mfrow)
-            n.dothi
-            if (length(goc) < 50) {
-                plot(goc, col = "blue", main = paste("Singh", 
-                  n, "fuzzy set"), type = "o", pch = 16, ylim = c(min(c(goc, 
-                  dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                  xlab = "index", ylab = "data")
-                lines(dubao, col = "red", type = "o", pch = 18)
-                legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                  col = c("blue", "red"), lty = c(1, 1), pch = c(16, 
-                    18), cex = 1/n.dothi)
-            }
-            if (length(goc) > 49) {
-                plot(goc, col = "blue", main = paste("Singh", 
-                  n, "fuzzy set"), type = "l", ylim = c(min(c(goc, 
-                  dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                  xlab = "index", ylab = "data")
-                lines(dubao, col = "red", type = "l")
-                legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                  col = c("blue", "red"), lty = c(1, 1), , cex = 1/n.dothi)
-            }
-            if (n.dothi == 1) {
-                k.ve <- par()$yaxp
-                h = (k.ve[2] - k.ve[1])/k.ve[3]
-                for (i in 0:k.ve[3]) abline(h = k.ve[1] + h * 
-                  i, lty = 3, col = "gray")
-            }
-        }
-        if (type == "Heuristic") {
-            goc <- DB3[, 2]
-            dubao <- DB3[, 4]
-            n.dothi <- prod(par()$mfrow)
-            n.dothi
-            if (length(goc) < 50) {
-                plot(goc, col = "blue", main = paste("Heuristic", 
-                  n, "fuzzy set"), type = "o", pch = 16, ylim = c(min(c(goc, 
-                  dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                  xlab = "index", ylab = "data")
-                lines(dubao, col = "red", type = "o", pch = 18)
-                legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                  col = c("blue", "red"), lty = c(1, 1), pch = c(16, 
-                    18), cex = 1/n.dothi)
-            }
-            if (length(goc) > 49) {
-                plot(goc, col = "blue", main = paste("Heuristic", 
-                  n, "fuzzy set"), type = "l", ylim = c(min(c(goc, 
-                  dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                  xlab = "index", ylab = "data")
-                lines(dubao, col = "red", type = "l")
-                legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                  col = c("blue", "red"), lty = c(1, 1), cex = 1/n.dothi)
-            }
-            if (n.dothi == 1) {
-                k.ve <- par()$yaxp
-                h = (k.ve[2] - k.ve[1])/k.ve[3]
-                for (i in 0:k.ve[3]) abline(h = k.ve[1] + h * 
-                  i, lty = 3, col = "gray")
-            }
-        }
-        if (type == "Chen-Hsu") {
-            if (!is.null(bin)) {
-                n = dim(D.ch)[1]
-                goc <- DB4[, 2]
-                dubao <- DB4[, 4]
-                n.dothi <- prod(par()$mfrow)
-                n.dothi
-                if (length(goc) < 50) {
-                  plot(goc, col = "blue", main = paste("Chen-Hsu", 
-                    n, "fuzzy set"), type = "o", pch = 16, ylim = c(min(c(goc, 
-                    dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                    xlab = "index", ylab = "data")
-                  lines(dubao, col = "red", type = "o", pch = 18)
-                  legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                    col = c("blue", "red"), lty = c(1, 1), pch = c(16, 
-                      18), cex = 1/n.dothi)
-                }
-                if (length(goc) > 49) {
-                  plot(goc, col = "blue", main = paste("Chen-Hsu", 
-                    n, "fuzzy set"), type = "l", ylim = c(min(c(goc, 
-                    dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
-                    xlab = "index", ylab = "data")
-                  lines(dubao, col = "red", type = "l")
-                  legend("bottomright", "(x,y)", c("ts", "forecast"), 
-                    col = c("blue", "red"), lty = c(1, 1), cex = 1/n.dothi)
-                }
-                if (n.dothi == 1) {
-                  k.ve <- par()$yaxp
-                  h = (k.ve[2] - k.ve[1])/k.ve[3]
-                  for (i in -2:(k.ve[3] + 2)) abline(h = k.ve[1] + 
-                    h * i, lty = 3, col = "gray")
-                }
-            }
-        }
-    }
-    MO
+    
+if (plot == TRUE) {
+ve<-0
+
+if (type == "Chen") {
+goc <- DB1[, 2]
+dubao <- DB1[, 4]
+ve<-1
+}
+
+
+if (type == "Singh") {
+goc <- DB2[, 2]
+dubao <- DB2[, 4]
+ve<-1
+}
+
+
+if (type == "Heuristic") {
+goc <- DB3[, 2]
+dubao <- DB3[, 4]  
+ve<-1       
+}
+
+if (type == "Chen-Hsu") {
+if (!is.null(bin)) {
+n = dim(D.ch)[1]
+goc <- DB4[, 2]
+dubao <- DB4[, 4]
+ve<-1
+}
+}
+
+
+if(ve==1){
+if(c(par()$mfrow)[1]>2 | c(par()$mfrow)[2]>2 )
+warning("Graph only paint when: c(par()$mfrow)[1] < 3 & c(par()$mfrow)[1] < 3")
+else{
+n.dothi <- sum(par()$mfrow)
+n.dothi<- n.dothi/2
+if(n.dothi==1) n.dothi<-1/0.8
+main.plot<-c("Actual series vs forecated series by",paste(type,"model of", n, "fuzzy set"))
+
+if (length(goc) < 50) {
+if(c(par()$mfrow)[2]==1)
+plot(goc, col = "blue", main =paste(main.plot[1],main.plot[2]),cex.main=0.8, type = "o", pch = 15, 
+ylim = c(min(c(goc, dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
+xlab = "point", ylab = "data",bty="l")
+
+if(c(par()$mfrow)[2]==2)
+plot(goc, col = "blue", main =c(main.plot[1],main.plot[2]),cex.main=0.8, type = "o", pch = 15, 
+ylim = c(min(c(goc, dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
+xlab = "point", ylab = "data",bty="l")
+
+lines(dubao, col = "red", type = "o", pch = 17)
+legend("topleft", "(x,y)", c("Actual","Forecasted"), ncol=2,
+col = c("blue", "red"), lty = c(1, 1), pch = c(15,17), cex = 1/n.dothi,box.lty=0)
+}
+
+if (length(goc) > 49) {
+
+if(c(par()$mfrow)[2]==1)
+plot(goc, col = "blue", main =paste(main.plot[1],main.plot[2]),cex.main=0.8, type = "l", 
+ylim = c(min(c(goc, dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
+xlab = "point", ylab = "data",bty="l")
+
+if(c(par()$mfrow)[2]==2)
+plot(goc, col = "blue", main =c(main.plot[1],main.plot[2]),cex.main=0.8, type = "l",
+ylim = c(min(c(goc, dubao), na.rm = 1), max(c(goc, dubao), na.rm = 1)), 
+xlab = "point", ylab = "data",bty="l")
+
+lines(dubao, col = "red", type = "l")
+legend("topleft", "(x,y)", c("Actual","Forecasted"),ncol=2, 
+col = c("blue", "red"), lty = c(1, 1), cex = 1/n.dothi,box.lty=0)
+}
+
+if(grid==1)grid.on(v=0)
+
+}
+}
+}
+MO
 }
